define([ 'jquery', 'underscore', 'backbone', 'tpls', 'c3', 'events', 'context', 'moment'],
    function ($, _, Backbone, Templates, c3, Events, Context, Moment) {
        return Backbone.View.extend({
            template: Templates["content/output/viz"],
            errorTemplate: Templates["content/output/viz/error"],
            initialize: function () {
                this.listenTo(Events, "query:fetched", this.drawChart);
                this.listenTo(Events, "settings:update", this.updateSettings);
                //this.listenTo(Events, "chart:load", this.loadChart);
            },
            getPlottableData: function (data) {

                // default values
                var plottableData = [],
                    columns = [];

                if (data.length > 0) {

                    // get plottable columns
                    columns = this.getPlottableColumns(data);

                    _.each(columns, function (item) {
                        
                        // get all values by column name
                        var columnData = _.pluck(data, item.column);

                        // check if column is date and needs to be parsed
                        if (item.type === "date") {
                            columnData = _.map(columnData, function (val) {
                                return Moment(val);
                            });
                        }

                        var escapedColumnName = item.column.replace(/([^a-zA-Z0-9_-])/g, '');
                        columnData.unshift(escapedColumnName);
                        plottableData.push(columnData);
                    });
                }
                return {data: plottableData, columns: columns};
            },
            getPlottableColumns: function (data) {
                if (data.length > 0) {
                    // check first row for plottable data
                    var columns = _.map(data[0], function (v, k) {
                        if (this.isNumber(v)) {
                            return {type:"string", column: k};
                        }

                        var date = Moment(new Date(v));
                        if (date.isValid()) {
                            return {type:"date", column: k};
                        }
                    }, this);

                    return _.compact(columns);
                }
                return [];
            },
            isNumber: function (n) {
                return !isNaN(parseFloat(n)) && isFinite(n);
            },
            updateSettings: function () {
                if (this.data) {
                    this.drawChart(this.data, true);
                } else {
                    console.warn("can't update settings, because data is not set");
                }
            },
            drawChart: function (data, skipDataProp) {
                this.data = data;
                var plottable = this.getPlottableData(data.rows);

                // if data has changed
                if (skipDataProp !== true) {
                    // update columns in settings view
                    this.parent.subviews["settings"].updateColumns(plottable);
                }

                // store plottable data
                this.plottableData = plottable.data;

                if (this.plottableData.length > 0) {

                    // default settings
                    var dataSettings = {},
                        axisSettings = {};

                    // get xaxis and update chart settings
                    var xAxis = this.parent.subviews["settings"].getXAxis();
                    if (!_.isEmpty(xAxis)) {

                        // set xaxis settings
                        dataSettings.x = xAxis;

                        // find column in plottable columns
                        var column = _.findWhere(plottable.columns, {column: xAxis})

                        // check if xAxis could be timeseries
                        if (!_.isEmpty(column) && column.type === "date") {
                            axisSettings.x = {
                                type: 'timeseries',
                                tick: {
                                    format: '%Y-%m-%d',
                                    culling: {
                                        max: 5
                                    }
                                }
                            };
                        }
                    }

                    // get chart type from settings view
                    dataSettings.type = this.parent.subviews["settings"].getChartMode();

                    // set chart data
                    dataSettings.columns = this.plottableData;

                    // create chart
                    this.chart = c3.generate({
                        bindto: this.$("#chart").get(0),
                        size: {
                            width: 700,
                            height: 400
                        },
                        data: dataSettings,
                        color: {
                            pattern: ["#8EB746", "#3E2C52", "#E84B0C", "#D3D3D3"]
                        },
                        legend: {
                            show: this.parent.subviews["settings"].getShowLegend()
                        },
                        axis: axisSettings
                    });

                    // store settings in settings view
                    this.parent.subviews["settings"].xAxis = xAxis;

                    console.log("c3 chart", this.chart);

                } else {
                    this.$("#chart").html(this.errorTemplate({error: "Not enough data to draw a chart"}));
                }
            },
            loadChart: function (chart) {
                console.log("viz.loadChart", chart);
                this.updateSettings();
            },
            render: function () {
                this.$el.html(this.template());

                if (!_.isEmpty(Context.result)) {
                  this.drawChart(Context.result);
                }
                return this;
            }
        });
    });
